var nkn = {
    // This is just a placeholder.
};


// nkn.hide hides HTML content by adding class name "nkn-hide" or replacing class name "nkn-show" with "nkn-hide".
// This function accepts either a string (the HTML id) or the actual DOM node you want to hide.
nkn.hide = function(someHTML) {

    var myHTML = null;

    if (typeof(someHTML) == "string") {
        myHTML = document.getElementById(someHTML);
    }
    else if (someHTML instanceof Element) {
        myHTML = someHTML;
    }

    if (myHTML) {
        if (myHTML.classList.contains('nkn-show')) {
            myHTML.classList.remove("nkn-show");
        } 
        myHTML.classList.add("nkn-hide");
    }
}


// nkn.show displays hidden HTML content by replacing class name "nkn-hide" with "nkn-show".
// This function accepts either a string (the HTML id) or the actual DOM node you want to show.
nkn.show = function(someHTML) {

    var myHTML = null;

    if (typeof(someHTML) == "string") {
        myHTML = document.getElementById(someHTML);
    }
    else if (someHTML instanceof Element) {
        myHTML = someHTML;
    }

    if (myHTML) {
        if (myHTML.classList.contains('nkn-hide')) {
            myHTML.classList.remove("nkn-hide");
        } 
        myHTML.classList.add("nkn-show");
    }
}


// nkn.showParentElement finds someElement's nearest parent with class name "nkn-hide" and calls nkn.show with this element.
// If a parent element with classname "nkn-show" instead of "nkn-hide" is encountered, we do nothing (element already shown).
nkn.showParentElement = function(someElement){

    function findHiddenElement(el) {
        while ((el = el.parentElement) && !(el.classList.contains("nkn-hide") || el.classList.contains("nkn-show"))) ;
        return el;
    }

    var myHiddenElement = findHiddenElement(someElement);
    if (myHiddenElement){
        nkn.show(myHiddenElement);
    }
};


// nkn.hideParentElement finds someElement's nearest parent with classname "nkn-show" and calls nkn.hide with this element.
// If a parent element with classname "nkn-hide" instead of "nkn-show" is encountered, we do nothing (element already hidden).
nkn.hideParentElement = function(someElement){

    function findShownElement(el) {
        while ((el = el.parentElement) && !(el.classList.contains("nkn-hide") || el.classList.contains("nkn-show"))) ;
        return el;
    }

    var myShownElement = findShownElement(someElement);
    if (myShownElement){
        nkn.hide(myShownElement);
    }
};


// nkn.findParentElement returns el's nearest parent element with the given CSS class.
nkn.findParentElement = function(el, className) {
    while ((el = el.parentElement)  &&  !el.classList.contains(className));
    return el;
}


// nkn.clickHandler is a global click handler which checks if event.target or
// any of event.target's parent elements has a defined data-nkn-role attribute.
// If an nkn-role is defined we call the corresponding handler function.
// IMPORTANT: if a client application attaches a custom click handler to an
// element with nkn-role or a parent element thereof, custom JavaScript may be required.
nkn.clickHandler = function(event) {

    // findNKNTarget returns el if el has a defined data-nkn-role attribute
    // or the nearest parent element which has a defined data-nkn-role attribute.
    function findNKNTarget(el) {
        while (!el.dataset.nknRole) {
            if (el.parentElement) {
                el = el.parentElement;
            } else {
                return null;
            }
        }
        return el;
    }
    // Find nkn-role of click target, if any.
    var nknRole = null;
    var nknTarget = findNKNTarget(event.target);
    if (nknTarget) {
        nknRole = nknTarget.getAttribute("data-nkn-role");
    }

    // If there are any dropdown open, close it.
    if(!nknRole || (nknTarget.parentElement.classList.contains("nkn-open") == false && nknTarget)){
        var myDropdowns = document.getElementsByClassName("nkn-dropdown");
        for (var i = 0; i < myDropdowns.length; i++) {
            nkn.closeDropdown(myDropdowns[i]);
        } 
    }

    // If there is no nkn-role, nothing further to do.
    if (!nknRole) {
        return false;
    }

    // If click target has a defined nkn-role, call corresponding handler function.
    switch (nknRole) {
        case "accordion-toggle":
            nkn.toggleAccordionItem(nknTarget);
            break;
        case "open-modal":
            nkn.openModal(nknTarget.getAttribute("data-nkn-target"));
            break;
        case "close-modal":
            nkn.closeModal(nknTarget);
            break;
        case "hamburger-toggle":
            nkn.toggleHamburger(nknTarget);
            break;
        case "navbar-toggle":
            nkn.toggleNavbar(nknTarget);
            break;
        case "dropdown-toggle":
            nkn.dropdown(nknTarget);
            break;
    }

    return false;
}


// closeRemainingAccordion closes all items in someAccordion except for someItem.
// This is typically used if you want an accordion by design to open only a single item at a time.
nkn.closeRemainingAccordion = function(someAccordion, someItem) {
    var myItems = someAccordion.getElementsByClassName("nkn-accordion-item");
    for (var indx = 0; indx < myItems.length; indx++) {
        if (myItems[indx] != someItem) {
            myItems[indx].classList.remove("nkn-open");
        }
    }
 }
  
  
 // nkn.toggleAccordionItem opens or closes an item in an accordion.
 nkn.toggleAccordionItem = function(toggle) {
    var myAccordion = nkn.findParentElement(toggle, "nkn-accordion");
    var myItem = nkn.findParentElement(toggle, "nkn-accordion-item");
    if (myItem) {
        if (myItem.classList.contains("nkn-open")) {
            myItem.classList.remove("nkn-open");
        } else {
            myItem.classList.add("nkn-open");
            if (myAccordion.classList.contains("nkn-open-only-one")) {
                this.closeRemainingAccordion(myAccordion, myItem);
            }
        }
    }
 }
 // closeDropDown closes a specified dropdown.
nkn.closeDropdown = function(myDropdown) {
    if (myDropdown) {
        if (myDropdown.classList.contains('nkn-open')) {
            myDropdown.classList.remove("nkn-open");
        } 
    }
}

// dropdown opens or closes a dropdown by adding or removing CSS class nkn-open to the HTML element with class nkn-dropdown.
// toggle is the element which was clicked to trigger this function, ie. any element with data-nkn-role = dropdown-toggle.
nkn.dropdown = function(toggle) {  
    var myDropdown = nkn.findParentElement(toggle, "nkn-dropdown");
   
    function setDropPosition() {

        var contentDropdown = myDropdown.getElementsByClassName("nkn-dropdown-content")[0];
        contentDropdown.style = null;
        
        function distanceFromBottom() {
            const windowHeight = window.innerHeight;
            const distanceFromWindowTop = contentDropdown.getBoundingClientRect().top;
            const elementHeight = contentDropdown.clientHeight;
            return windowHeight - distanceFromWindowTop - elementHeight;
        }

        // Vertical position.
        const canDropDown = contentDropdown.clientHeight < distanceFromBottom();
        // if we don't have space to show the dropdown content at the bottom of the dropdown, change postion to start from bottom side.
        if (!canDropDown) {
            contentDropdown.style.bottom = toggle.clientHeight + "px";
        }
        else{ 
            contentDropdown.style.bottom = null;
        }

        // Horizontal position.
        var myDropdownPosition = myDropdown.parentNode.getBoundingClientRect();
        
        // If we are in scroll, use getBoundingClientRect to set correct position for dropdown content.
        if((myDropdownPosition.right >= 0) && (myDropdownPosition.left <= window.innerHeight)){
            contentDropdown.style.right = myDropdownPosition.right - toggle.clientHeight + "px";
        }else{
            contentDropdown.style.right = myDropdown.offsetRight -toggle.clientHeight + "px";
        }

        // If dropdown doesn't fit correctly in default position, change position to start from right side.
        var widths = window.innerWidth - myDropdown.getBoundingClientRect().left - contentDropdown.clientWidth;
        if(widths < 0){
            contentDropdown.style.right = "1px";
            contentDropdown.style.left = null;
        }else{
            contentDropdown.style.right = null;
        }
    } 

    if (myDropdown) {
        if (myDropdown.classList.contains('nkn-open')) {
            myDropdown.classList.remove("nkn-open");                      
        } else {
            myDropdown.classList.add("nkn-open");   
            setDropPosition();      
        }
    }
}
// closeHamburger closes a "hamburger" menu by removing CSS class nkn-open.
// Clients usually do not need to call this directly, when opening a menu we automatically add "close" click handlers.
nkn.closeHamburger = function(e) {
    var myMenu = nkn.findParentElement(e.target, "nkn-hamburger-menu");
    if (myMenu) {
        myMenu.classList.remove("nkn-open");
    }
}


// toggleHamburger opens or closes a "hamburger" menu by adding or removing CSS class nkn-open to the element with class nkn-hamburger-menu.
// toggle is the element which was clicked to trigger this function, ie. any element with data-nkn-role = hamburger-toggle.
// When opening a "hamburger" menu, we add click handlers to each LI element to close the menu when clicked.
nkn.toggleHamburger = function(toggle) {
    var myMenu = nkn.findParentElement(toggle, "nkn-hamburger-menu");
    if (myMenu) {
        if (myMenu.classList.contains('nkn-open')) {
            myMenu.classList.remove("nkn-open");
        } else {
            myMenu.classList.add("nkn-open");
            var myLinks = myMenu.querySelectorAll("LI");
            for (var indx = 0; indx < myLinks.length; indx++) {
                myLinks[indx].addEventListener("click", nkn.closeMenu);
            }
        }
    }
}
// closeModal closes a modal by removing CSS class nkn-open from the modal's container.
nkn.closeModal = function(closeButton) {
    var myModalContainer = nkn.findParentElement(closeButton, "nkn-modal-container");
    if (myModalContainer) {
        myModalContainer.classList.remove("nkn-open");
    }
}


// openModal opens a modal by adding CSS class nkn-open to the modal's container.
// The caller may pass either a string with the modal's DOM element id or the modal's actual DOM element.
nkn.openModal = function(someHTML) {

    var myHTML = null;

    if (typeof(someHTML) == "string") {
        myHTML = document.getElementById(someHTML);
    }
    else if (someHTML instanceof Element) {
        myHTML = someHTML;
    }

    var myModalContainer = nkn.findParentElement(myHTML, "nkn-modal-container");
    if (myModalContainer) {
        myModalContainer.classList.add("nkn-open");
    }
}




// closeNavbar closes a responsive navbar by removing CSS class nkn-open.
// Clients usually do not need to call this directly, when opening a responsive navbar we automatically add "close" click handlers.
nkn.closeNavbar = function(e) {
    var myMenu = nkn.findParentElement(e.target, "nkn-navbar");
    if (myMenu) {
        myMenu.classList.remove("nkn-open");
    }
}


// toggleNavbar opens or closes a responsive navbar by adding or removing CSS class nkn-open to the  element with class nkn-navbar.
// toggle is the element which was clicked to trigger this function, ie. any element with data-nkn-role = navbar-toggle.
// When opening a responsive navbar, we add click handlers to each LI element to close the navbar when clicked for LI elements that has class nkn-nav-item.
nkn.toggleNavbar = function(toggle) {
    var myMenu = nkn.findParentElement(toggle, "nkn-navbar");
    if (myMenu) {
        if (myMenu.classList.contains('nkn-open')) {
            myMenu.classList.remove("nkn-open");
        } else {
            myMenu.classList.add("nkn-open");
            var myLinks = myMenu.querySelectorAll("LI");
            for (var indx = 0; indx < myLinks.length; indx++) {
                if (myLinks[indx].classList.contains("nkn-nav-item") ) {
                    if(myLinks[indx].classList.contains("nkn-dropdown")) {
                        var myDropdown = myLinks[indx];
                        var contentDropdown = myDropdown.getElementsByClassName("nkn-dropdown-content")[0];
                        contentDropdown.addEventListener("click", nkn.closeNavbar);
                    } else {

                        myLinks[indx].addEventListener("click", nkn.closeNavbar);
                    }

                }
            }
        }
    }
}

// When page is ready we add a click event listener to the HTML body.
// If the event target has a data-nkn-role attribute, we'll do something with the event.
document.addEventListener("DOMContentLoaded", function(event) { 
    document.body.addEventListener("click", nkn.clickHandler);
});


